/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ObservationSetSupport.java,v 2abe70262508 2010/04/29 21:41:05 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation.spi;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.Serializable;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationSet;
import it.tidalwave.bluebill.observation.ObservationSet.Event;
import it.tidalwave.bluebill.observation.ObservationSet.Listener;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public abstract class ObservationSetSupport implements ObservationSet, Serializable, Externalizable
  {
    private static final String CLASS = ObservationSetSupport.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    private static final long serialVersionUID = 546345436345345343L;

    private transient List<Listener> listeners = new ArrayList<Listener>();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static interface ListenerTask
      {
        public void run (@Nonnull Listener listener);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void addListener (final @Nonnull Listener listener)
      {
        if (listeners == null) listeners = new ArrayList<Listener>(); // FIXME: readExternal seems not to work
        listeners.add(listener);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void removeListener (final @Nonnull Listener listener)
      {
        if (listeners == null) listeners = new ArrayList<Listener>(); // FIXME: readExternal seems not to work
        listeners.remove(listener);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void writeExternal (final @Nonnull ObjectOutput oo)
      throws IOException
      {
        // listeners are transient
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void readExternal (final @Nonnull ObjectInput oi)
      throws IOException, ClassNotFoundException
      {
        listeners = new ArrayList<Listener>();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void fireObservationAdded (final @Nonnull Observation observation)
      {
        final Event event = new Event(this, observation, null);
        fire(new ListenerTask()
          {
            public void run (final @Nonnull Listener listener)
              {
                listener.notifyObservationAdded(event);
              }
          });
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void fireObservationRemoved (final @Nonnull Observation oldObservation)
      {
        final Event event = new Event(this, null, oldObservation);
        fire(new ListenerTask()
          {
            public void run (final @Nonnull Listener listener)
              {
                listener.notifyObservationRemoved(event);
              }
          });
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void fireObservationChanged (final @Nonnull Observation observation,
                                           final @Nonnull Observation oldObservation)
      {
        final Event event = new Event(this, observation, oldObservation);
        fire(new ListenerTask()
          {
            public void run (final @Nonnull Listener listener)
              {
                listener.notifyObservationChanged(event);
              }
          });
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void fireNotifyCleared()
      {
        final Event event = new Event(this, null, null);
        fire(new ListenerTask()
          {
            public void run (final @Nonnull Listener listener)
              {
                listener.notifyCleared(event);
              }
          });
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void fire (final @Nonnull ListenerTask listenerTask)
      {
        for (final Listener listener : new ArrayList<Listener>(listeners))
          {
            try
              {
                listenerTask.run(listener);
              }
            catch (Throwable t)
              {
                logger.warning("During event notification: " + t);
                logger.throwing("fire()", CLASS, t);
              }
          }
      }
  }
